/*
 * cmdu_ackq.h
 * CMDU pending response and ack queue management
 *
 * Copyright (C) 2020 IOPSYS Software Solutions AB. All rights reserved.
 *
 * Author: anjan.chanda@iopsys.eu
 *
 * See LICENSE file for license related information.
 *
 */

#ifndef CMDU_ACKQ_H
#define CMDU_ACKQ_H


#include "hlist.h"
#include <easy/easy.h>
#include <wifi.h>

#define CMDU_BACKLOG_MAX	128
#define MAC_ADDR_HASH(a)	(a[0] ^ a[1] ^ a[2] ^ a[3] ^ a[4] ^ a[5])
// TODO: improve hash func
#define cmdu_ackq_hash(t, m, o)		\
	((MAC_ADDR_HASH(o) ^ (t) ^ (m)) & (CMDU_BACKLOG_MAX - 1))

/* struct holds a pending cmdu with 'mid' from 'origin' */
struct cmdu_ackq_entry {
	uint16_t type;
	uint16_t mid;
	uint8_t origin[6];
	struct hlist_node hlist;

	uint32_t ageing_time;    /* in msecs */
	struct timeval ageing_tmo;
};

/* queue of cmdus for which response is pending */
struct cmdu_ackq {
	pthread_mutex_t qlock;

	/* hashtable of pending cmdu_ackq_entry */
	struct hlist_head table[CMDU_BACKLOG_MAX];

	int pending_cnt;
	atimer_t ageing_timer;
	struct timeval next_tmo;
};

/* struct holds a STA events */
struct sta_eventlog_entry {
	char ifname[16];
	uint8_t mac[6];
	uint8_t bssid[6];
	struct wifi_sta sta;
	atimer_t t;
	struct sta_eventlog *sv;		// Pointer to hold sta eventlog queue address
	struct hlist_node hlist;
};

/* queue for STA events */
struct sta_eventlog {
	pthread_mutex_t evlock;
	int active_sta_cnt;
	struct hlist_head start;
};

extern int cmdu_ackq_init(void *q);
extern void cmdu_ackq_free(void *q);
extern void cmdu_ackq_reset(void *q);
extern int cmdu_ackq_dequeue(void *q, uint16_t type, uint16_t mid, uint8_t *src);
extern int cmdu_ackq_enqueue(void *q, uint16_t type, uint16_t mid, uint8_t *dest,
			     uint32_t timeout);

extern int sta_eventlog_init(void *sta_log);
extern struct sta_eventlog_entry *sta_eventlog_lookup(void *sta_log, uint8_t *mac);
extern int sta_eventlog_enqueue(void *sta_log, const char *ifname, uint8_t *mac);
extern int sta_eventlog_dequeue(void *sta_log, uint8_t *mac);

#endif /* CMDU_ACKQ_H */
