/*
 * delm.h
 * WiFi Data Elements plugin private header file.
 *
 * Copyright (C) 2020 IOPSYS Software Solutions AB. All rights reserved.
 *
 * Author: anjan.chanda@iopsys.eu
 *
 * See LICENSE file for license related information.
 *
 */

#ifndef DELM_H
#define DELM_H


#include "cmdu_ackq.h"

#ifdef WFA_DELM_STANDALONE

#define WIFI_DEVICE_MAX_NUM	4
#define WIFI_IFACE_MAX_NUM	16

struct wifi_netdev {
	char radio[16];
	struct {
		char name[16];
		int mode;
	} iface[WIFI_IFACE_MAX_NUM];
};

enum {
	WIFI_IFACE_FH,
	WIFI_IFACE_BK,
};

#endif /* WFA_DELM_STANDALONE */

#define MAP_PROFILE_1	1
#define MAP_PROFILE_2	2

typedef uint32_t object_t;

#define OBJECT_INVALID	(uint32_t)-1

#define WFADATAD_AGENT_OBJECT		"wifi.dataelements"
#define WFADATAD_COLLECTOR_OBJECT	"wifi.dataelements.collector"

struct delm_config {
	uint32_t refresh_int;
	bool is_collector;
	bool is_sap_mode;
	uint32_t profile;
	struct ubus_context *uctx;
};

#define DELM_OBJECT_PLACEHOLDER			\
	struct ubus_context *ctx;		\
	struct ubus_object obj;			\
	struct ubus_event_handler ieee1905_cmdu;\
	struct ubus_event_handler ieee1905_ev;	\
	struct ubus_event_handler topology_ev;	\
	struct ubus_event_handler sta_ev;



#define MAX_IEEE1905_INTERFACE_NUM	16

struct ieee1905_ifnames {
	uint32_t num;
	char ifname[MAX_IEEE1905_INTERFACE_NUM][16];
};

struct delm_private {
	struct wifi_netdev ifs[WIFI_DEVICE_MAX_NUM];
	struct delm_config cfg;
	atimer_t t;
	int getter_running;
	uint8_t cntlr_id[6];		/* controller id */
	char cntlr_al_bridge[16];	/* controller al bridge */
	bool dm_available;
	struct wifi_data_element *dm;
	struct wifi_data_element *dmi;	/* intermediate */
	struct wifi_sta_events *sv;	/* sta events */
	int pending;
	object_t ieee1905;
	object_t map1905;
	object_t topology;
	uint8_t ieee1905_macaddr[6];	/* self node macaddr */
	struct ieee1905_ifnames iifs;
	struct cmdu_ackq cmdu_ack_q;
	struct sta_eventlog assoc_stalog;
	DELM_OBJECT_PLACEHOLDER
};

int delm_add_device(struct wifi_network_device *remote_data, struct wifi_network_device *data);

extern int delm_publish_object(struct delm_private *p, const char *objname);
extern int delm_register_events(struct delm_private *p);
extern uint32_t delm_lookup_object(struct delm_private *p, const char *objname);


extern int delm_dump_all(struct delm_private *p, struct blob_buf *bb);
extern int delm_event_dump(struct delm_private *p, struct blob_buf *bb);
extern int delm_collect_node(struct delm_private *p, struct wifi_network_device *dev);
extern struct wifi_network_device *delm_get_origin_dev(
		struct delm_private *p, uint8_t *origin);

extern int delm_get_deagents(struct delm_private *p, int *num, uint8_t **macaddrs);
extern int delm_get_ieee1905_interfaces(struct delm_private *p);

extern void *delm_getter_run(void *data);
extern void *delm_collector_run(void *data);

extern void delm_free_dm(struct wifi_data_element *dm);
extern void delm_update_datamodel(struct delm_private *p);

extern char *get_date(time_t *t, char *tbuf);
extern char *get_timestamp(time_t *t, char *tbuf);

// extern void *get_local_device_data(struct delm_private *p);
// extern int delm_add_neighbor_node(void *priv, void *remote_dev);
// extern int delm_del_neighbor_node(void *priv, uint8_t *macaddr);
extern void free_dev_data(struct wifi_network_device *dev);
extern int self_node_query(struct delm_private *p);
extern void refresh_sta_timer_cb(atimer_t *t);
extern int delm_get_assoc_sta(struct delm_private *p, uint8_t *sta_mac,
		int status_code, char *ev_tsp, uint8_t *bssid);
extern int delm_get_unassoc_sta(struct delm_private *p, uint8_t *sta_mac,
		int reason_code, char *ev_tsp);
extern int delm_fill_sta_bssid(struct delm_private *p, uint8_t *sta_mac, uint8_t *bssid);
extern int delm_update_stalog_data(void *sta_log, const char *ifname, uint8_t *mac);
extern int prepare_autoconfig_search(struct delm_private *priv, uint8_t band);

#endif /* DELM_H */
