/*
 * delm_collector.c
 * WiFi Data Elements Collector main
 *
 *
 * Author: anjan.chanda@iopsys.eu
 *
 * Copyright (C) 2020 IOPSYS Software Solutions AB. All rights reserved.
 *
 * See LICENSE file for license related information.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pthread.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <netinet/in.h>

#include <easy/easy.h>
#include <wifi.h>

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <libubus.h>

#include <map1905/map2.h>
#include <map1905/maputils.h>

#ifndef WFA_DELM_STANDALONE
#include <map-agent/mapagent.h>
#include <map-agent/timer.h>
#else
#include "timer.h"
#endif

#include "wfa_delm.h"
#include "delm.h"


void *delm_collector_run(void *arg)
{
	struct delm_private *p = (struct delm_private *)arg;
	int num = 0;
	uint8_t *macaddrs = NULL;
	struct wifi_data_element *dm;
	int ret;
	int i;


	if (p->getter_running) {
		fprintf(stderr, "Collector already running...\n");
		return NULL;
	}

	fprintf(stdout, "In collector --->\n");

	p->getter_running = 1;

	ret = delm_get_deagents(p, &num, &macaddrs);
	if (ret || num == 0) {
		err("%s: num agents = %d\n", __func__, num);
		goto out;
	}

	dm = calloc(1, sizeof(struct wifi_data_element) +
			num * sizeof(struct wifi_network_device));
	if (!dm) {
		free(macaddrs);
		goto out;
	}

	get_timestamp(NULL, dm->network.tsp);
	dm->network.num_devices = num;
	p->dmi = dm;
	p->pending = num;

	for (i = 0; i < num; i++) {
		memcpy(&dm->network.device[i].macaddr, &macaddrs[i * 6], 6);
		fprintf(stderr, "%s: [%d] device-macaddr = " MACFMT "\n",
			__func__, i, MAC2STR(dm->network.device[i].macaddr));

		ret |= delm_collect_node(p, &dm->network.device[i]);
	}

	if ((num > 0) && macaddrs)
		free(macaddrs);

	return NULL;

out:
	p->getter_running = 0;

	return NULL;
}
