/*
 * delm_dumper.c
 * WiFi Data Elements json objects dump functions
 *
 * Copyright (C) 2020 IOPSYS Software Solutions AB. All rights reserved.
 *
 * See AUTHORS file for author and contributors.
 *
 * See LICENSE file for license related information.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <json-c/json.h>
#include <libubox/blobmsg.h>
#include <libubox/blobmsg_json.h>
#include <libubox/uloop.h>
#include <libubox/ustream.h>
#include <libubox/utils.h>
#include <libubus.h>

#include <easy/easy.h>
#ifndef WFA_DELM_STANDALONE
#include <map-agent/mapagent.h>
#include <map-agent/timer.h>
#else
#include "timer.h"
#endif

#include "wfa_delm.h"
#include "delm.h"

static int remove_spec_char(unsigned char *out, int out_len,
		unsigned char *src, int src_len)
{
	int i;
	int copy_len = 0;

	memset(out, 0, out_len);
	copy_len = out_len > src_len ? src_len : out_len;

	for (i = 0; i < copy_len; i++) {
		if (src[i] == '\n') {
			out[i] = '\0';
			return 0;
		}
		else
			out[i] = src[i];
	}

	out[i] = '\0';
	return 0;
}

int delm_dump_station(struct wifi_sta_element *sta, struct blob_buf *bb)
{
	char macaddr[17] = {0};
	void *arr;
	void *t;
	int i;
	size_t out_len;
	unsigned char caps_str[16];

	hwaddr_ntoa(sta->macaddr, macaddr);
	blobmsg_add_string(bb, "MACAddress", macaddr);
	blobmsg_add_string(bb, "TimeStamp", sta->tsp);
	blobmsg_add_u32(bb, "SignalStrength", sta->rssi);
	out_len = sizeof(caps_str);
	memset(caps_str, 0, sizeof(caps_str));
	base64_encode(&sta->caps.ht, sizeof(sta->caps.ht), caps_str, &out_len);
	blobmsg_add_string(bb, "HTCapabilities", (char *)caps_str);
	out_len = sizeof(caps_str);
	memset(caps_str, 0, sizeof(caps_str));
	base64_encode(sta->caps.vht, sizeof(sta->caps.vht), caps_str, &out_len);
	blobmsg_add_string(bb, "VHTCapabilities", (char *)caps_str);
	blobmsg_add_u32(bb, "EstMACDataRateDownlink", sta->dl_est_thput);
	blobmsg_add_u32(bb, "EstMACDataRateUplink", sta->ul_est_thput);
	blobmsg_add_u32(bb, "LastDataUplinkRate", sta->ul_rate);
	//blobmsg_add_string(bb, "Airtime", sta->dl_utilization);  //TODO
	blobmsg_add_u64(bb, "UtilizationTransmit", sta->dl_utilization);
	blobmsg_add_u64(bb, "UtilizationReceive", sta->ul_utilization);
	blobmsg_add_u32(bb, "LastConnectTime", sta->conn_time);
	//blobmsg_add_string(bb, "IPV4Address", sta->ipaddr.addr);
	//blobmsg_add_string(bb, "IPV6Address", sta->ipaddr.addr); //TODO
	blobmsg_add_string(bb, "Hostname", sta->hostname);
	blobmsg_add_u64(bb, "BytesSent", sta->tx_bytes);
	blobmsg_add_u64(bb, "BytesReceived", sta->rx_bytes);
	blobmsg_add_u32(bb, "PacketsSent", sta->tx_pkts);
	blobmsg_add_u32(bb, "PacketsReceived", sta->rx_pkts);
	blobmsg_add_u32(bb, "ErrorsSent", sta->tx_errors);
	blobmsg_add_u32(bb, "ErrorsReceived", sta->rx_errors);
	blobmsg_add_u32(bb, "NumberOfMeasureReports", sta->num_bcn_reports);

	arr = blobmsg_open_array(bb, "Measurementreport");
	for (i = 0; i < sta->num_bcn_reports; i++) {
		t = blobmsg_open_table(bb, "");
		//delm_dump_sta_bcnreport(sta->bcn_reportlist, bb);
		blobmsg_close_table(bb, t);
	}
	blobmsg_close_array(bb, arr);

	return 0;
}

int delm_dump_bss(struct wifi_bss_element *bss, struct blob_buf *bb)
{
	char bssid[17] = {0};
	void *arr;
	void *t;
	int i;
	size_t out_len;
	unsigned char est_str[16];


	hwaddr_ntoa(bss->bssid, bssid);
	blobmsg_add_string(bb, "BSSID", bssid);
	blobmsg_add_string(bb, "SSID", (char *)bss->ssid);
	blobmsg_add_string(bb, "TimeStamp", bss->tsp);
	blobmsg_add_u8(bb, "Enabled", bss->enabled);
	blobmsg_add_u32(bb, "LastChange", bss->uptime);
	blobmsg_add_u32(bb, "NumberofSTA", bss->num_stations);
	out_len = sizeof(est_str);
	memset(est_str, 0, sizeof(est_str));
	base64_encode(bss->est_wmm_be, 3, est_str, &out_len);
	blobmsg_add_string(bb, "EstServiceParametersBE", (char *)est_str);
	out_len = sizeof(est_str);
	memset(est_str, 0, sizeof(est_str));
	base64_encode(bss->est_wmm_bk, 3, est_str, &out_len);
	blobmsg_add_string(bb, "EstServiceParametersBK", (char *)est_str);
	out_len = sizeof(est_str);
	memset(est_str, 0, sizeof(est_str));
	base64_encode(bss->est_wmm_vo, 3, est_str, &out_len);
	blobmsg_add_string(bb, "EstServiceParameterVO", (char *)est_str);
	out_len = sizeof(est_str);
	memset(est_str, 0, sizeof(est_str));
	base64_encode(bss->est_wmm_vi, 3, est_str, &out_len);
	blobmsg_add_string(bb, "EstServieParameterVI", (char *)est_str);
	blobmsg_add_u64(bb, "UnicastBytesReceived", bss->rx_ucast_bytes);
	blobmsg_add_u64(bb, "UnicastBytesSent", bss->tx_ucast_bytes);
	blobmsg_add_u64(bb, "MulticastBytesReceived", bss->rx_mcast_bytes);
	blobmsg_add_u64(bb, "MulticastBytesSent", bss->tx_mcast_bytes);
	blobmsg_add_u64(bb, "BroadcastBytesReceived", bss->rx_bcast_bytes);
	blobmsg_add_u64(bb, "BroadcastBytesSent", bss->tx_bcast_bytes);

	arr = blobmsg_open_array(bb, "STAList");
	for (i = 0; i < bss->num_stations; i++) {
		t = blobmsg_open_table(bb, "");
		delm_dump_station(bss->stalist + i, bb);
		blobmsg_close_table(bb, t);
	}
	blobmsg_close_array(bb, arr);

	return 0;
}

int delm_dump_scanres_nbr(struct wifi_scanres_neighbor_element *nbr,
							struct blob_buf *bb)
{
	char nbr_bssid[17] = {0};

	hwaddr_ntoa(nbr->bssid, nbr_bssid);
	blobmsg_add_u32(bb, "StationCount", nbr->num_stations);
	blobmsg_add_u32(bb, "ChannelUtilization", nbr->utilization);
	blobmsg_add_string(bb, "BSSID", nbr_bssid);
	blobmsg_add_string(bb, "SSID", nbr->ssid);
	blobmsg_add_u32(bb, "SignalStrengh", nbr->rssi);
	blobmsg_add_u32(bb, "ChannelBandwidth", nbr->bw);

	return 0;
}

int delm_dump_channel_scanres(struct wifi_scanres_channel_element *ch,
							struct blob_buf *bb)
{
	void *arr;
	void *t;
	int i;

	/*fprintf(stderr, "%s:%d channel = %d num_nbrs = %d   nbrlist = %p\n",
			__func__, __LINE__,
			ch->channel, ch->num_neighbors, ch->nbrlist); */

	blobmsg_add_string(bb, "TimeStamp", ch->tsp);
	blobmsg_add_u32(bb, "Channel", ch->channel);
	blobmsg_add_u32(bb, "NumberofNeighbors", ch->num_neighbors);
	blobmsg_add_u32(bb, "Noise", ch->anpi);
	blobmsg_add_u32(bb, "Utilization", ch->utilization);

	arr = blobmsg_open_array(bb, "NeighborList");
	for (i = 0; i < ch->num_neighbors; i++) {
		t = blobmsg_open_table(bb, "");
		delm_dump_scanres_nbr(ch->nbrlist + i, bb);
		blobmsg_close_table(bb, t);
	}
	blobmsg_close_array(bb, arr);

	return 0;
}

int delm_dump_opclass_scanres(struct wifi_scanres_opclass_element *op,
							struct blob_buf *bb)
{
	void *arr;
	void *t;
	int i;


	/* fprintf(stderr, "%s:%d opclass = %d num_channels_scanned = %d\n",
		__func__, __LINE__, op->opclass, op->num_channels_scanned); */

	blobmsg_add_u32(bb, "OperatingClass", op->opclass);
	blobmsg_add_u32(bb, "NumberOfChannelScans", op->num_channels_scanned);

	arr = blobmsg_open_array(bb, "ChannelScanList");
	for (i = 0; i < op->num_channels_scanned; i++) {
		t = blobmsg_open_table(bb, "");
		delm_dump_channel_scanres(op->channel_scanlist + i, bb);
		blobmsg_close_table(bb, t);
	}
	blobmsg_close_array(bb, arr);

	return 0;
}

int delm_dump_scanresult(struct wifi_scanres_element *sres, struct blob_buf *bb)
{
	void *arr;
	void *t;
	int i;


	/* fprintf(stderr, "%s:%d sres->num_opclass_scanned = %d\n",
			__func__, __LINE__, sres->num_opclass_scanned); */

	blobmsg_add_string(bb, "TimeStamp", sres->tsp);
	blobmsg_add_u32(bb, "NumberOfOpClassScans", sres->num_opclass_scanned);

	arr = blobmsg_open_array(bb, "OpClassScanList");
	for (i = 0; i < sres->num_opclass_scanned; i++) {
		t = blobmsg_open_table(bb, "");
		delm_dump_opclass_scanres(sres->opclass_scanlist + i, bb);
		blobmsg_close_table(bb, t);
	}
	blobmsg_close_array(bb, arr);

	return 0;
}

int delm_dump_unassoc_sta(struct wifi_unassoc_sta_element *usta,
							struct blob_buf *bb)
{
	char macaddr[17] = {0};

	hwaddr_ntoa(usta->macaddr, macaddr);
	blobmsg_add_string(bb, "MACAddress", macaddr);
	blobmsg_add_u32(bb, "SignalStrength", usta->rssi);

	return 0;
}

int delm_dump_backhaul_sta(struct wifi_backhaul_element *bksta,
						struct blob_buf *bb)
{
	char macaddr[17] = {0};

	hwaddr_ntoa(bksta->macaddr, macaddr);
	blobmsg_add_string(bb, "MACAddress", macaddr);
	return 0;
}

int delm_dump_curr_opclass(struct wifi_opclass_current_element *curr,
							struct blob_buf *bb)
{
	blobmsg_add_string(bb, "TimeStamp", curr->tsp);
	blobmsg_add_u32(bb, "Channel", curr->channel);
	blobmsg_add_u32(bb, "Class", curr->id);
	blobmsg_add_u32(bb, "TxPower", curr->txpower);

	return 0;
}

int delm_dump_supp_opclass(struct wifi_opclass_supported_element *supp,
							struct blob_buf *bb)
{
	void *arr;
	//void *t;
	int i;


	blobmsg_add_u32(bb, "Class", supp->id);
	blobmsg_add_u32(bb, "MaxTxPower", supp->max_txpower);
	blobmsg_add_u32(bb, "NumberOfNonOperChan", supp->num_exclude_channels);

	/* non-operable channels */
	arr = blobmsg_open_array(bb, "NonOperable");
	for (i = 0; i < supp->num_exclude_channels; i++) {
		//t = blobmsg_open_table(bb, "");
		blobmsg_add_u32(bb, "", (int)supp->exclude_chanlist[i]);
		//blobmsg_close_table(bb, t);
	}
	blobmsg_close_array(bb, arr);

	return 0;
}

int delm_dump_radio(struct wifi_radio_element *radio, struct blob_buf *bb,
		bool is_sap_mode)
{
	char radio_id[17] = {0};
	void *t, *t2;
	void *arr;
	int i;
	size_t out_len;
	unsigned char caps_str[16];
	unsigned char caps_str1[16];
	int str1_len = sizeof(caps_str1);


	hwaddr_ntoa(radio->macaddr, radio_id);
	blobmsg_add_string(bb, "ID", radio_id);
	blobmsg_add_u32(bb, "Noise", radio->anpi);
	blobmsg_add_u32(bb, "NumberOfBSS", radio->num_bss);
	blobmsg_add_u8(bb, "Enabled", radio->enabled);
	blobmsg_add_u32(bb, "NumberOfCurrOpClass", radio->num_curr_opclass);
	blobmsg_add_u32(bb, "NumberOfUnassocSta", radio->num_unassoc_sta);
	blobmsg_add_u32(bb, "Utilization", radio->total_utilization);
	blobmsg_add_u32(bb, "Transmit", radio->tx_utilization);
	blobmsg_add_u32(bb, "ReceiveSelf", radio->rx_utilization);
	blobmsg_add_u32(bb, "RecieveOther", radio->other_utilization);

	/* bss */
	arr = blobmsg_open_array(bb, "BSSList");
	for (i = 0; i < radio->num_bss; i++) {
		t = blobmsg_open_table(bb, "");
		delm_dump_bss(radio->bsslist + i, bb);
		blobmsg_close_table(bb, t);
	}
	blobmsg_close_array(bb, arr);

	/* capabilities { supp-opclass, ht/vht/he caps } */
	t = blobmsg_open_table(bb, "Capabilites");
	blobmsg_add_u32(bb, "NumberOfOpClass", radio->num_supp_opclass);
	arr = blobmsg_open_array(bb, "OperatingClasses");
	for (i = 0; i < radio->num_supp_opclass; i++) {
		t2 = blobmsg_open_table(bb, "");
		delm_dump_supp_opclass(radio->supp_opclass + i, bb);
		blobmsg_close_table(bb, t2);
	}
	blobmsg_close_array(bb, arr);
	out_len = sizeof(caps_str);
	memset(caps_str, 0, sizeof(caps_str));
	base64_encode(&radio->caps.ht, sizeof(radio->caps.ht), caps_str, &out_len);
	remove_spec_char(caps_str1, str1_len, caps_str, out_len);
	blobmsg_add_string(bb, "HTCapabilities", (char *)caps_str1);
	out_len = sizeof(caps_str);
	memset(caps_str, 0, sizeof(caps_str));
	base64_encode(radio->caps.vht, sizeof(radio->caps.vht), caps_str, &out_len);
	remove_spec_char(caps_str1, str1_len, caps_str, out_len);
	blobmsg_add_string(bb, "VHTCapabilities", (char *)caps_str1);
	blobmsg_close_table(bb, t);

	/* current opclass */
	arr = blobmsg_open_array(bb, "CurrentOperatingClasses");
	for (i = 0; i < radio->num_curr_opclass; i++) {
		t = blobmsg_open_table(bb, "");
		delm_dump_curr_opclass(radio->curr_opclass + i, bb);
		blobmsg_close_table(bb, t);
	}
	blobmsg_close_array(bb, arr);

	/* scan results */
	arr = blobmsg_open_array(bb, "ScanResultList");
	for (i = 0; i < radio->num_scanresult; i++) {
		t = blobmsg_open_table(bb, "");
		delm_dump_scanresult(radio->scanlist + i, bb);
		blobmsg_close_table(bb, t);
	}
	blobmsg_close_array(bb, arr);

	/* unassociated stas */
	arr = blobmsg_open_array(bb, "UnassociatedStaList");
	for (i = 0; i < radio->num_unassoc_sta; i++) {
		t = blobmsg_open_table(bb, "");
		delm_dump_unassoc_sta(radio->unassoc_stalist + i, bb);
		blobmsg_close_table(bb, t);
	}
	blobmsg_close_array(bb, arr);

	if (!is_sap_mode) {
		/* backhaul sta */
		t = blobmsg_open_table(bb, "BackhaulSta");
		delm_dump_backhaul_sta(&radio->bksta, bb);
		blobmsg_close_table(bb, t);
	}

	return 0;
}

int delm_dump_device(struct wifi_network_device *dev, struct blob_buf *bb,
		bool is_sap_mode)
{
	char dev_id[17] = {0};
	void *arr;
	void *t;
	int i;

	hwaddr_ntoa(dev->macaddr, dev_id);
	blobmsg_add_string(bb, "ID", dev_id);
	blobmsg_add_u32(bb, "NumberOfRadios", dev->num_radios);
	blobmsg_add_u32(bb, "CollectionInterval", dev->collect_int);

	arr = blobmsg_open_array(bb, "RadioList");
	for (i = 0; i < dev->num_radios; i++) {
		t = blobmsg_open_table(bb, "");
		delm_dump_radio(dev->radiolist + i, bb, is_sap_mode);
		blobmsg_close_table(bb, t);
	}
	blobmsg_close_array(bb, arr);

	return 0;
}

int delm_dump_all(struct delm_private *p, struct blob_buf *bb)
{
	struct wifi_network_device *dev = NULL;
	struct wifi_data_element *dm = p->dm;
	struct wifi_network *net;
	char cntlr_id[17] = {0};
	char dstr[32] = {0};
	char tsp[32] = {0};
	void *t, *t2, *t3;
	void *arr, *arr2;
	int i;
	int available_nodes;


	if (!p->dm_available)
		return -1;

	if (!dm)
		return -1;

	net = &dm->network;
	available_nodes = net->num_devices;

	if (p->cfg.is_collector) {
		for (i = 0; i < net->num_devices; i++) {
			if (!net->device[i].is_valid_node)
				available_nodes--;
		}
	}

	blobmsg_add_string(bb, "date", get_date(NULL, dstr));
	blobmsg_add_string(bb, "version", "0.2");	// dm->version
	blobmsg_add_string(bb, "description", "WFA Data Elements");
	blobmsg_add_string(bb, "TimeStamp", get_timestamp(NULL, tsp));
	blobmsg_add_string(bb, "name", "wifi");

	arr = blobmsg_open_array(bb, "data");
	t = blobmsg_open_table(bb, "");

	t2 = blobmsg_open_table(bb, "wfa-dataelements:Network");

	blobmsg_add_string(bb, "ID", net->id);
	blobmsg_add_u32(bb, "NumberOfDevices", available_nodes);
	hwaddr_ntoa(net->cntlr_id, cntlr_id);
	if (!p->cfg.is_sap_mode)
		blobmsg_add_string(bb, "ControllerID", cntlr_id);

	blobmsg_add_string(bb, "TimeStamp", net->tsp);

	arr2 = blobmsg_open_array(bb, "DeviceList");
	for (i = 0; i < net->num_devices; i++) {
		dev = &net->device[i];
		if (p->cfg.is_collector && !dev->is_valid_node)
			continue;

		t3 = blobmsg_open_table(bb, "");
		delm_dump_device(dev, bb, p->cfg.is_sap_mode);
		blobmsg_close_table(bb, t3);
	}
	blobmsg_close_array(bb, arr2);
	blobmsg_close_table(bb, t2);

	blobmsg_close_table(bb, t);
	blobmsg_close_array(bb, arr);

	return 0;
}

int delm_dump_assoc_event(struct wifi_assoc_events *assoc_ev, struct blob_buf *bb)
{
	char macstr[18] = {0};
	size_t out_len;
	unsigned char caps_str[16];
	void *t1, *t2, *t3;
	unsigned char caps_str1[16];
	int str1_len = sizeof(caps_str1);

	t1 = blobmsg_open_table(bb, "");
	blobmsg_add_string(bb, "eventTime", assoc_ev->tsp);
	t2 = blobmsg_open_table(bb, "wfa-dataelements:AssociationEvent");
	t3 = blobmsg_open_table(bb, "AssocData");
	memset(macstr, 0, sizeof(macstr));
	hwaddr_ntoa(assoc_ev->bssid, macstr);
	blobmsg_add_string(bb, "BSSID", macstr);
	memset(macstr, 0, sizeof(macstr));
	hwaddr_ntoa(assoc_ev->macaddr, macstr);
	blobmsg_add_string(bb, "MACAddress", macstr);
	blobmsg_add_u32(bb, "StatusCode", assoc_ev->status_code);
	out_len = sizeof(caps_str);
	memset(caps_str, 0, sizeof(caps_str));
	base64_encode(&assoc_ev->caps.ht, sizeof(assoc_ev->caps.ht), caps_str, &out_len);
	remove_spec_char(caps_str1, str1_len, caps_str, out_len);
	blobmsg_add_string(bb, "HTCapabilities", (char *)caps_str1);
	out_len = sizeof(caps_str);
	memset(caps_str, 0, sizeof(caps_str));
	base64_encode(assoc_ev->caps.vht, sizeof(assoc_ev->caps.vht), caps_str, &out_len);
	remove_spec_char(caps_str1, str1_len, caps_str, out_len);
	blobmsg_add_string(bb, "VHTCapabilities", (char *)caps_str1);
	blobmsg_close_table(bb, t3);
	blobmsg_close_table(bb, t2);
	blobmsg_close_table(bb, t1);

	return 0;
}

int delm_dump_disassoc_event(struct wifi_disassoc_events *disassoc_ev, struct blob_buf *bb)
{
	char macstr[18] = {0};
	void *t1, *t2, *t3;

	t1 = blobmsg_open_table(bb, "");
	blobmsg_add_string(bb, "eventTime", disassoc_ev->tsp);
	t2 = blobmsg_open_table(bb, "wfa-dataelements:DisassociationEvent");
	t3 = blobmsg_open_table(bb, "DisassocData");
	memset(macstr, 0, sizeof(macstr));
	hwaddr_ntoa(disassoc_ev->bssid, macstr);
	blobmsg_add_string(bb, "BSSID", macstr);
	memset(macstr, 0, sizeof(macstr));
	hwaddr_ntoa(disassoc_ev->sta_data.macaddr, macstr);
	blobmsg_add_string(bb, "MACAddress", macstr);
	blobmsg_add_u32(bb, "ReasonCode", disassoc_ev->reason_code);
	blobmsg_add_u32(bb, "BytesSent", disassoc_ev->sta_data.tx_bytes);
	blobmsg_add_u32(bb, "BytesReceived", disassoc_ev->sta_data.rx_bytes);
	blobmsg_add_u32(bb, "PacketsSent", disassoc_ev->sta_data.tx_pkts);
	blobmsg_add_u32(bb, "PacketsReceived", disassoc_ev->sta_data.rx_pkts);
	blobmsg_add_u32(bb, "ErrorsSent", disassoc_ev->sta_data.tx_errors);
	blobmsg_add_u32(bb, "ErrorsReceived", disassoc_ev->sta_data.rx_errors);
	blobmsg_add_u32(bb, "RetransCount", disassoc_ev->sta_data.rtx_pkts);
	blobmsg_close_table(bb, t3);
	blobmsg_close_table(bb, t2);
	blobmsg_close_table(bb, t1);

	return 0;
}

int delm_event_dump(struct delm_private *p, struct blob_buf *bb)
{
	int i;
	void *arr;
	struct wifi_sta_events *sv = p->sv;

	arr = blobmsg_open_array(bb, "notification");
	for (i = 0; i < sv->num_assoc_events; i++) {
		delm_dump_assoc_event(sv->assoclist + i, bb);
	}

	for (i = 0; i < sv->num_disassoc_events; i++) {
		delm_dump_disassoc_event(sv->disassoclist + i, bb);
	}
	blobmsg_close_array(bb, arr);

	return 0;
}
